/**
 * Copyright 2006 (C) Jeffrey Palm
 */

// --------------------------------------------------
// Variables
// --------------------------------------------------

var xmlhttp;              // retrieves XML from my site
var type;                 // "band" | "location"
var bandOrLocationName;   // holds the band or location name for the title
var MAX_ITEMS = 15;       // max number of items in the list
var currentIndex;         // current index to search from
var currentSearch;        // current search term
var theShows;             // the current array of shows
var HIDE_CALLOUT = false; // whether we hide the callout

// --------------------------------------------------
// Initialize the menu
// --------------------------------------------------

pluginHelper.onAddCustomMenuItems = function(menu) {
  menu.AddItem("Reset",0,gadgetReset);
  menu.AddItem("Test suite", 0, testSuite);
}

// --------------------------------------------------
// Methods
// --------------------------------------------------

/**
 * Void -> Void
 */
function viewOnopen() {
  gadgetReset();
}

/**
 * Void -> Void
 */
function gadgetReset() {
  setTitle("Please enter an artist or city & state");
  loading.visible = false;
  xmlhttp = false; 
  buttonLookup.enabled = true;
  textSource.value = "bloc party";
  mainContent.value = "";
  if (HIDE_CALLOUT) {
    imgCallout.visible = false;
    mainContent.visible = false;
  }
  buttonMore.visible = false;
  buttonBack.visible = false;
}

/**
 * Void -> XMLHttp
 */
function createXMLThingy() {
  try {
    xmlhttp = new XMLHttpRequest();
    if (xmlhttp) return xmlhttp;
  } catch(e) {
    try {
      xmlhttp = new ActiveXObject("Microsoft.XMLHTTP");
      if (xmlhttp) return xmlhttp;
    } catch(e) {}
  }
  return xmlhttp;
}


/**
 * String -> Void: Sets the title at the top of the callout
 */
function setTitle(str) {
  labelTitle.innerText = str;
}

/**
 * String -> Void: Adds to the title at the top of the callout
 */
function appendTitle(str) {
  labelTitle.innerText += str;
}

/**
 * Void -> Void: looks up the text in 'textSource' and displays
 *               it in 'mainContent'
 */
function lookup() {

  currentIndex = 0;
  theShows = false;

  if(xmlhttp && xmlhttp.readyState != 0 && xmlhttp.readyState != 4) return;

  var txt = textSource.value;
  currentSearch = txt;

  doLookUp(txt);
}

function doLookUp(txt) {

  // create the XML guy (or gal)
  xmlhttp = createXMLThingy();
  loading.visible = true;
  buttonLookup.enabled = false;
  bandOrLocationName = txt;
  setTitle("looking up " + txt + "...");
  var input = txt;
  //
  // remove white space around comma
  //
  input = input.replace(/,\s+/g,",");
  input = input.replace(/\s+,/g,",");
  input = input.replace(/\s/g,"+");
  input = escape(input);
  //
  // Construct the URL
  //
  var getURL;
  if ((icomma = input.indexOf("%2C")) == -1) {
    // -- band
    getURL = "http://jeffpalm.com/cgi-bin/pollstar2.pl?band="+input;
    type = "band";
  } else {
    // -- location
    city = input.substring(0,icomma);
    state = input.substr(icomma+3);
    getURL = "http://jeffpalm.com/cgi-bin/pollstar.pl?city=" + city + "&state=" + state;
    type = "location";
  }
  //
  // Make the request
  //
  xmlhttp.open("GET",getURL,true);
  xmlhttp.onreadystatechange = lookupDone;
  xmlhttp.send("");
}

/**
 * Void -> Void: Propogates 'mainContent' with the results of the XML query
 */
function lookupDone() {
  if(xmlhttp.readyState == 4 && xmlhttp.status == 200) {
    xml = xmlhttp.responseXML.documentElement;
    showShows(theShows = xml.getElementsByTagName("show"));
  } else if( xmlhttp.readyState == 4 ){
    loading.visible = false;
    buttonLookup.enabled = true;
    alert(strError1 + xmlhttp.status + "\n" + strError2 + "\n" + strError3);
  }
}

function showShows(shows) {
  try {
    
    if (HIDE_CALLOUT) {
      imgCallout.visible = true;
      mainContent.visible = true;
    }

    buttonMore.visible = false;
    buttonBack.visible = false;

    removeAllElements();

    needMore = false;
    MAX = currentIndex+MAX_ITEMS;
      
    if (type == "band") {

      setTitle("shows for " + bandOrLocationName);

      for (i=currentIndex; i<shows.length; i++) {

	if (i>MAX) {
	  needMore = true;
	  break;
	}

	s = shows[i];

	date = trimDate(s.getAttribute("date"));
	city = s.getAttribute("city");
	venue = s.getAttribute("venue");
	venueLink = s.getAttribute("venueLink");

	str = date + " in " + city + " @ " + venue;
	addContentItem(str);
      }
    } else {

      setTitle("shows in " + bandOrLocationName);

      shows = xml.getElementsByTagName("show");
      lastDate = 0;
      for (i=currentIndex; i<shows.length; i++) {

	if (i>MAX) {
	  needMore = true;
	  break;
	}

	s = shows[i];

	date = trimDate(s.getAttribute("date"));
	artist = s.getAttribute("artist");
	venue = s.getAttribute("venue");
	venueLink = s.getAttribute("venueLink");
	str = date + ": " + artist + " @ " + venue;
	addContentItem(str);
      }
    }

    // add to the title
    if (shows.length > MAX_ITEMS) {
      appendTitle(" - " + (currentIndex/MAX_ITEMS+1) + " of " + (roundInt(shows.length/MAX_ITEMS+1)));
    }

    if (needMore) {
      buttonMore.visible = true;
      buttonMore.onclick = function() {
	currentIndex += MAX_ITEMS;
	//doLookUp(currentSearch);
	showShows(theShows);
      };
    }
    needLess = currentIndex>0;
    if (needLess) {
      buttonBack.visible = true;
      buttonBack.onclick = function() {
	currentIndex -= MAX_ITEMS;
	//doLookUp(currentSearch);
	showShows(theShows);
      };
    }

  } catch (e) {
    alert(e);
    loading.visible = false;
    buttonLookup.enabled = true;
  }
  loading.visible = false;
  buttonLookup.enabled = true;
}



/**
 * Void -> Void: Removes all the elements in 'mainContent'
 */
function removeAllElements() {
  mainContent.value = "";
}


/**
 * String -> Void: Adds a String item to 'mainContent'
 */
function addContentItem(item) {
  var N = 40;
  if (item.length>N) item = item.substring(0,N) + "...";
  mainContent.value += item + "\n";
}

/**
 * String -> String: removes year from date
 */
function trimDate(date) {
  // 12/23/34 -> 12/23
  return date.substring(0,5);
}

// --------------------------------------------------
// Test suite
// --------------------------------------------------

/**
 * String -> Void: Runs one test.
 */
function testRunOne(text) {
  textSource.value = text;
  lookup();
}

/**
 * Void -> Void: Runs a silly stupid 'test suite'
 */
function testSuite() {
  names = ["radiohead",
	   "bloc party",
	   "new orleans, la",
	   "new orleans,la",
	   "new orleans ,la",
	   "new orleans ,LA"
  ];
  function loop(i) {
    testRunOne(names[i]);
    if (i<names.length) loop(i+1);
  }
  loop(0);
}

/**
 * double -> String(int)
 */
function roundInt(i) {
  s = String(i);
  return s.replace(/\..*/,"");
}
